package ru.dbotthepony.kstarbound.world

/**
 * Предоставляет доступ к чанку и его соседям
 *
 * В основном для использования в местах, где нужен не мир, а определённый чанк мира,
 * и при этом координаты проверяются относительно чанка и могут спокойно выйти за его пределы,
 * с желанием получить тайл из соседнего чанка
 */
open class TileView(
	override val pos: ChunkPos,
	open val center: ITileChunk?,

	open val right: ITileChunk?,
	open val top: ITileChunk?,
	open val topRight: ITileChunk?,
	open val topLeft: ITileChunk?,

	open val left: ITileChunk?,
	open val bottom: ITileChunk?,
	open val bottomLeft: ITileChunk?,
	open val bottomRight: ITileChunk?,
) : ITileChunk {
	override fun getTile(x: Int, y: Int): TileState {
		if (x in 0 ..CHUNK_SIZE_FF) {
			if (y in 0 ..CHUNK_SIZE_FF) {
				return center?.getTile(x, y) ?: TileState.EMPTY
			}

			if (y < 0) {
				return bottom?.getTile(x, y + CHUNK_SIZE) ?: TileState.EMPTY
			} else {
				return top?.getTile(x, y - CHUNK_SIZE) ?: TileState.EMPTY
			}
		}

		if (x < 0) {
			if (y in 0 ..CHUNK_SIZE_FF) {
				return left?.getTile(x + CHUNK_SIZE, y) ?: TileState.EMPTY
			}

			if (y < 0) {
				return bottomLeft?.getTile(x + CHUNK_SIZE, y + CHUNK_SIZE) ?: TileState.EMPTY
			} else {
				return topLeft?.getTile(x + CHUNK_SIZE, y - CHUNK_SIZE) ?: TileState.EMPTY
			}
		} else {
			if (y in 0 ..CHUNK_SIZE_FF) {
				return right?.getTile(x - CHUNK_SIZE, y) ?: TileState.EMPTY
			}

			if (y < 0) {
				return bottomRight?.getTile(x - CHUNK_SIZE, y + CHUNK_SIZE) ?: TileState.EMPTY
			} else {
				return topRight?.getTile(x - CHUNK_SIZE, y - CHUNK_SIZE) ?: TileState.EMPTY
			}
		}
	}
}

class MutableTileView(
	pos: ChunkPos,
	override val center: IMutableTileChunk?,

	override val right: IMutableTileChunk?,
	override val top: IMutableTileChunk?,
	override val topRight: IMutableTileChunk?,
	override val topLeft: IMutableTileChunk?,

	override val left: IMutableTileChunk?,
	override val bottom: IMutableTileChunk?,
	override val bottomLeft: IMutableTileChunk?,
	override val bottomRight: IMutableTileChunk?,
) : TileView(pos, center, right, top, topRight, topLeft, left, bottom, bottomLeft, bottomRight), IMutableTileChunk {
	override fun setTile(x: Int, y: Int, value: TileState) {
		TODO()
	}
}
